#!/bin/bash

# $Id: slis_update,v 3.1 2003/05/27 15:20:15 bzizou Exp $ #

# This script is part of the SLIS Project initiated by the CARMI-Internet
# (Acadmie de Grenoble - France 38).
# Ce script fait partie du projet SLIS dmarr par le CARMI-Internet
# (Acadmie de Grenoble - France 38).
#
# SLIS : Serveur de communications Linux pour l'Internet Scolaire.
# Copyright (C) 1998-2000 Bruno Bzeznik
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#    You should have received a copy of the GNU General Public License
#    along with this program (For example ./COPYING);
#    if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
#    Cambridge, MA 02139, USA.
#
# Please send all comments and bug reports by electronic mail to:
#   Bruno Bzeznik <Bruno@ac-grenoble.fr>
# or to <slis@ac-grenoble.fr>
#
# Envoyez vos suggestions et reports de bugs par e-mail 
#   Bruno Bzeznik <Bruno@ac-grenoble.fr>
# ou  <slis@ac-grenoble.fr>
#

. /home/hadmin/slis.conf
VERSION=`cat /etc/version_slis_num`

# Updates can be freezed because of rsync or psql waiting infinetely
# for the end of a dead request. So, there's a locking system a bit particular.

MAX_AGE=36000	# This is the maximum number of seconds you think
		# an update must leave before suspecting it to be
		# freezed.

RETRY_TIME=360	# This is the max number of seconds you wait after
		# having "refreshed" a freezed update for starting
		# the new one.

TEST_TIMEOUT=180 # This is the maximum time (s) to wait for the test
                 # connection that is done before everything 

# Locking
LOCK_OUTPUT=`lockfile -1 -r 3 -l $MAX_AGE -s 0 /var/lock/slis_update 2>&1`; LOCK_RC=$?
 
# If unable to lock (recent lock file found)
if [ "$LOCK_RC" != "0" ]
then
  logger -t "slis_update.cron[$PPID]" "ERROR: Unable to lock. Is another update running?"
  echo "ERROR: Unable to lock. Is another update active?" | mail "$SLISMASTER"
  exit 1
fi
 
# If a very old lock file is found
if [ "`echo $LOCK_OUTPUT | grep -i "forcing lock"`" != "" ]
then
  logger -t "slis_update.cron[$PPID]" "Warning: forcing lock and killing probably sleeping update"
 
  # Perhaps rsync or psql is freezed
  killall rsync 2> /dev/null
  killall psql 2> /dev/null
 
  # Try to lock another time while waiting for the previous update to finish
  lockfile -5 -r $RETRY_TIME /var/lock/slis_update ; LOCK_RC=$?
  if [ "$LOCK_RC" != "0" ]
  then
    logger -t "slis_update.cron[$PPID]" "ERROR: Second try for locking failed"
    echo "Error in slis_update.cron: Second try for locking failed" | mail "$SLISMASTER"
    exit 1
  fi
fi
 
# Test the connection
logger -t "slis_update.cron[$PPID]" "Downloading test files"
cd /tmp
rsync rsync://$RSYNC_HOST/$RSYNC_MODULE/TESTFILE* . &
RSYNC_PID=$!
RUNNING=0
declare -i c=0
while [ "$RUNNING" = "0" ]
do
  sleep 1
  let c=$c+1
  if [ $c  -ge $TEST_TIMEOUT ]
  then
    logger -t "slis_update.cron[$PPID]" "ERROR: TESTFILE* files could not be downloaded. Aborting updates."
    killall rsync
    /bin/rm -f TESTFILE* 
    rm -rf /var/lock/slis_update
    exit 2
  fi
  ps $RSYNC_PID >/dev/null;
  RUNNING=$?
done
logger -t "slis_update.cron[$PPID]" "Verifying checksums"
/usr/bin/md5sum -c TESTFILE.md5 >/dev/null 2>&1;
if [ "$?" = "0" ]
then
  logger -t "slis_update.cron[$PPID]" "Connection test OK, starting updates..."
  /bin/rm -f TESTFILE* 
else
  logger -t "slis_update.cron[$PPID]" "ERROR: connection test failed (incorrect md5sum). Aborting updates."
  /bin/rm -f TESTFILE* 
  rm -rf /var/lock/slis_update
  exit 2
fi

# Start the update
cd /tmp
rsync rsync://$RSYNC_HOST/$RSYNC_MODULE/slis_update-common-pre .
/bin/bash ./slis_update-common-pre
rsync rsync://$RSYNC_HOST/$RSYNC_MODULE/slis_update-$VERSION .
/bin/bash ./slis_update-$VERSION
rsync rsync://$RSYNC_HOST/$RSYNC_MODULE/slis_update-common-post .
/bin/bash ./slis_update-common-post
rsync rsync://$RSYNC_HOST/$RSYNC_MODULE/slis_update-academie-$VERSION .
/bin/bash ./slis_update-academie-$VERSION

# Remove the lockfile
rm -rf /var/lock/slis_update
